﻿define("commerce/CommerceModule", [
    "epi/_Module",
    "epi/dependency",
    "epi/routes",
    "epi-cms/store/CustomQueryEngine",

    "commerce/Application",
    "commerce/component/Enum",

    "commerce/conversion/CatalogEntryConverter"
], function (
    _Module,
    dependency,
    routes,
    CustomQueryEngine,

    Application,
    Enum,

    CatalogEntryConverter
) {

    return dojo.declare("commerce.CommerceModule", [_Module], {
        // summary: this is the initializer of CommerceModule.

        _settings: null,

        constructor: function (settings) {
            this._settings = settings;
        },

        initialize: function () {
            // summary:
            //		Initialize module
            //
            // description:
            //      Dependencies registered by this module are: 'commerce.Application'

            this.inherited(arguments);

            this.registerDependency("commerce.Application", new Application(this._settings));   // register a singleton object of CommerceApplication
            this.registerDependency("commerce.EnumContentType", new Enum("CatalogEntry", "CatalogNode", "Catalog"));       // register a singleton array of available CommerceType, work like an enum

            this._setupConverters(dependency);

            // Initialize stores
            this._initializeStores();
        },

        _initializeStores: function () {
            var registry = this.resolveDependency("epi.storeregistry");
            registry.create("commerce.objectstore", this._getRestPath("commercestructure"), { idProperty: "contentLink", queryEngine: CustomQueryEngine });
        },

        _getRestPath: function (name) {
            return routes.getRestPath({ moduleArea: "commerce", storeName: name });
        },

        _setupConverters: function (dependency) {
            var converterRegistry = this.resolveDependency("epi.objectconverterregistry"),
                catalogEntryConverter = new CatalogEntryConverter();

            catalogEntryConverter.registerConverter(converterRegistry);
        }
    });
});