﻿define([
    "dojo/_base/declare",
    "dojo/_base/Deferred",
    "dojo/_base/lang",
    "epi",
    "epi/dependency",
    "epi-cms/contentediting/ContentActionSupport",
    "epi/shell/command/_Command",
    "epi/shell/command/_SelectionCommandMixin",
    "epi/i18n!epi/cms/nls/episerver.cms.command"
], function (declare, Deferred, lang, epi, dependency, ContentActionSupport, _Command, _SelectionCommandMixin, res) {

    // module:
    //		epi/cms/command/DeleteContent
    // summary:
    //		Executes delete on the model for the item currently in the active selection.

    return declare([_Command, _SelectionCommandMixin], {

        // label: [readonly] String
        //		The action text of the command to be used in visual elements.
        label: res.movetotrash,

        // iconClass: [readonly] String
        //		The icon class of the command to be used in visual elements.
        iconClass: "epi-iconTrash",

        wasteBasketId: null,

        postscript: function () {
            this.inherited(arguments);

            this.wasteBasketId = this.wasteBasketId || dependency.resolve("epi.cms.Application").getWastebasketPage();
        },

        _execute: function () {
            // summary:
            //		Deletes the currently selected item.
            // tags:
            //		protected

            var item = this.selection.data[0].data,
                clipboard = this.clipboard && this.clipboard.data,
                source = clipboard && clipboard.length == 1 && clipboard[0],
                isAncestor = (source != null && source.data != null) ? this.model.isAncestor(item, source.data) : false;

            if (this.model && item) {
                Deferred.when(this.model.move ? this.model.move(item, this.wasteBasketId) : this.model.remove(item),
                    lang.hitch(this, function (response) {
                        if (response && response.isSuccess === true) {
                            // If the deleted item is in the clipboard(which is going to be pasted), or is an ancestor of that item,
                            // clear the clipboard
                            Deferred.when(isAncestor, lang.hitch(this, function (result) {
                                if (result === true) {
                                    this.clipboard.clear();
                                }
                            }));
                            if (this.model.onDeleted) {
                                this.model.onDeleted(item);
                            }
                        }
                    }));
            }
        },

        _onModelChange: function () {
            // summary:
            //		Updates canExecute after the model has been updated.
            // tags:
            //		protected

            // TODO: Handle Page Provider capabilities
            var model = this.model,
                selection = this.selection.data,
                canExecute = false;

            if (model && selection.length == 1) {
                var firstSelectedItem = selection[0];
                canExecute = firstSelectedItem.type == "epi.cms.contentdata" && this._validateDelete(firstSelectedItem.data);
            }

            this.set("canExecute", canExecute);
        },

        _validateDelete: function (contentItem) {
            // summary:
            //      Returns true if removing the item from its current location is allowed
            
            return !contentItem.isDeleted && this.model.canDelete(contentItem);
        }
    });
});
