﻿define([
// dojo
"dojo/_base/declare",
"dojo/_base/array",
"dojo/_base/lang",
"dojo/_base/connect",
"dojo/_base/Deferred",
"dojo/dom-geometry",
"dojo/store/Memory",
"dojo/aspect",

// dijit
"dijit/_Widget",
"dijit/layout/_LayoutWidget",
"dijit/_TemplatedMixin",
"dijit/_WidgetsInTemplateMixin",
"dijit/Tree",
"dijit/layout/BorderContainer",
"dijit/layout/ContentPane",
"dijit/form/ComboBox",
// epi
"epi",
"epi/dependency",
"epi/UriParser",
"epi/shell/command/_WidgetCommandProviderMixin",
"epi/shell/command/OptionCommand",
"epi/shell/widget/layout/ComponentContainer",
"epi/shell/widget/dialog/Alert",
"epi-cms/_ContentContextMixin",
"epi-cms/command/NewContent",
"epi-cms/component/ContentFilter",
"epi-cms/widget/SearchBox",
"epi/shell/widget/_FocusableMixin",
// commerce
"commerce/widget/CatalogNodeList",
"commerce/widget/CatalogEntryList",
"commerce/component/CommerceIdTranslator",
"commerce/widget/CatalogStoreModel",
// template
"dojo/text!./templates/Catalogs.html",
// resource
"epi/i18n!epi/cms/nls/commerce.components"

],

function (
// dojo
    declare,
    array,
    lang,
    connect,
    Deferred,
    domGeometry,
    Memory,
    aspect,

// dijit
    _Widget,
    _LayoutWidget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Tree,
    BorderContainer,
    ContentPane,
    ComboBox,
// epi
    epi,
    dependency,
    UriParser,
    _WidgetCommandProviderMixin,
    OptionCommand,
    ComponentContainer,
    Alert,
    _ContentContextMixin,
    NewContentCommand,
    ContentFilter,
    SearchBox,
    _FocusableMixin,
// commerce
    CatalogNodeList,
    CatalogEntryList,
    CommerceIdTranslator,
    CatalogStoreModel,
// template
    template,
// resource
    resCommerce

    ) {

    var menuTypes = {
        rootMenu: 0,
        catalogNodeMenu: 1,
        catalogEntryMenu: 2
    };

    var contentType = {
        catalogNodeType: 3,
        catalogEntryType: 4
    };

    return declare("commerce.component.Catalogs", [_Widget, _LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _WidgetCommandProviderMixin, _FocusableMixin, _ContentContextMixin], {

        _tree: null,

        resources: resCommerce,

        templateString: template,

        marketSelector: null,

        _copyMode: null,

        _currentCatalogNode: null,

        _catalogNodeMenuType: null,

        _application: null,

        showItem: 10,

        _defaultMarket: "DEFAULT",

        // _componentsController: [private] epi.shell.widget.Application
        _componentsController: null,

        _widgetResizeHandle: null,

        postMixInProperties: function () {
            // summary: 
            //    Overridden to get components controller.
            //
            // tags:
            //    protected

            this._componentsController = dependency.resolve("epi.shell.controller.Components");
            this._application = this._application || dependency.resolve("commerce.Application");
            this.inherited(arguments);


            if (!this.model) {
                this.model = new CatalogStoreModel();
            }
        },

        postCreate: function () {
            this.inherited(arguments);

            // E.g.: "cms/blocks" area, will return all search provider(s) that have area is "cms/blocks".
            // Will search using all search providers if passed blank area.
            // "Commerce" is area of Commerce ProductSearchProvider
            this.searchBox.set("area", "Commerce");
            this._setSearchRoot();
            this.searchBox.onItemAction = lang.hitch(this, function (searchResult) {
                // highlight selected catalogNode and catalogEntry

                var metaData = searchResult.metadata;
                this._highlightTreeAndList(metaData.id, metaData.parentId, metaData.parentType);
            });

            //before search, build query with getExpired = true and current market Id
            aspect.before(this.searchBox._searchBox, "_startSearch", lang.hitch(this, function (queryParams) {
                queryParams = this._buildQuery(queryParams, "true", this.marketSelector.item ? this.marketSelector.item.marketId : this._defaultMarket);
                return [queryParams];
            }));

            this.connect(this.catalogNodeList, "onCatalogNodeChanged", this._onCatalogNodeChanged);
            this.connect(this.catalogNodeList, "onResize", lang.hitch(this, this._onResize));
            this.connect(this.catalogEntryList, "onSelectionChange", this._onCatalogEntryChanged);
        },

        startup: function () {
            if (this._started) {
                return;
            }

            this.inherited(arguments);

            this._newCommand = new NewContentCommand({
                contentType: ContentFilter.BlockType,
                iconClass: "epi-iconForms",
                label: resCommerce.command.label,
                resources: resCommerce
            });

            // select current catalog entry if any
            Deferred.when(this.getCurrentContent(), lang.hitch(this, function (content) {
                this.contentContextChanged(lang.mixin(content, { id: content.contentLink }), this);
            }));

            this.connect(this.getParent(), "onResize", "_onResize");

            //region Market 
            Deferred.when(this.model.getAllMarkets(), lang.hitch(this, function (markets) {
                this.catalogNodeList.set("selectedMarketId", markets.defaultMarket.marketId);
                var marketStore = new Memory({ data: markets.allMarkets, idProperty: "marketId" });
                this.marketSelector = new ComboBox({
                    id: "marketSelector_" + this.id, //use the gadget id to make sure id of the market selector is unique
                    searchAttr: "marketName",
                    readonly: true,
                    item: markets.defaultMarket, //default value selected
                    store: marketStore
                }, "inputMarketSelect");
                this.marketSelector.startup();
                this.connect(this.marketSelector, "onChange", "_onMarketChange");
                this.catalogEntryList.marketSelector = this.marketSelector;
            }));
        },

        _buildQuery: function (originalQuery, getExpired, marketId) {
            //summary: 
            //          build query based on current context
            //tags:
            //          private
            query = dojo.string.trim(originalQuery);
            return dojo.string.substitute("${query}__${getExpired}__${marketId}", { query: query, getExpired: getExpired, marketId: marketId });
        },

        restore: function () {
            // summary: Restore component visuals
            this.resize();
        },

        _onResize: function () {

            this.resize({ h: domGeometry.getMarginBox(this.domNode).h });

            this.onResize();
        },

        onResize: function () {
            // summary:
            //    Raised after the ResizeHandle resized.
            //
            // tags:
            //    public callback
        },

        resize: function (size) {
            this.inherited(arguments);

            // limit the max height of CatalogNodeList container height to half size of this component
            // to avoid bug when too many subnode expanded make the EntryList disappear
            if (domGeometry.getMarginBox(this.catalogContainerNode.domNode).h > size.h / 2) {
                this.catalogContainerNode.resize({ h: size.h / 2 });
            }

            var searchBoxSize = domGeometry.getMarginBox(this.searchBoxContainer);
            this.container.resize({ h: size.h - searchBoxSize.h });
        },

        _onCatalogNodeChanged: function (catalognode) {
            this.catalogEntryList.set("selectedCatalogNode", catalognode.item);
            this._newCommand.set("model", catalognode.item);

            this._catalogNodeMenuType = catalognode.isRootNode ? menuTypes.rootMenu : menuTypes.catalogNodeMenu;
            this.toggleContext(this._catalogNodeMenuType);
        },

        _onCatalogEntryChanged: function (selection) {
            if (selection) {
                this.toggleContext(menuTypes.catalogEntryMenu);
            }
            else {
                this.toggleContext(this._catalogNodeMenuType ? this._catalogNodeMenuType : menuTypes.rootMenu);

                // The selection is null after delete action
                if (this.catalogEntryList.currentContext) {
                    this._highlightTreeAndList(this.catalogEntryList.currentContext.id, this.catalogEntryList.currentContext.parentLink);
                }
            }
        },

        contentContextChanged: function (context, callerData) {
            // summary:
            //    Called when the currently loaded content changes. I.e. a new content data object is loaded into the preview area.
            // tags:
            //    protected

            if (!context || !context.id ||
                !context.capabilities || !context.capabilities.isBlock) { // not a catalogEntry
                return;
            }

            // wait for hightlight catalognode contain this catalogEntry
            var currentContent = context;
            Deferred.when(this.catalogNodeList.highlightNode(currentContent.parentLink), lang.hitch(this, function () {
                this.catalogEntryList.set("currentContext", currentContent);
                this.catalogEntryList.highlightItem(currentContent.id);
            }));
        },

        toggleContext: function (type) {
            switch (type) {
                case menuTypes.rootMenu:
                    this.catalogEntryList.hideCommands();
                    this.catalogNodeList.showCommands(true);
                    break;
                case menuTypes.catalogNodeMenu:
                    this.catalogEntryList.hideCommands();
                    this.catalogNodeList.showCommands(false);
                    break;
                case menuTypes.catalogEntryMenu:
                    this.catalogEntryList.showCommands();
                    this.catalogNodeList.hideCommands();
                    break;
                default: break;
            }
        },

        _toggleMenuItem: function (menuHandler, menuItemName, isDisabled) {
            // summary:
            //    Enable or disable the menu item.
            // tags:
            //    private

            if (menuHandler && menuHandler["disableItem"]) {
                menuHandler["disableItem"](menuItemName, isDisabled);
            }
        },

        _setSearchRoot: function (id) {
            // summary:
            //    set the Root for search with SearchBox
            // id: string
            //      The contentLink of root for searching
            // tags:
            //    private

            var rootToSearch = id ? id : this._application.getCommerceRoot(); // use commerceRoot to search for everything
            this.searchBox.set("searchRoots", rootToSearch);
        },

        _highlightTreeAndList: function (entryId, treeNodeId, treeNodeType) {
            // summary:
            //		Highlight selected node on tree and selected entry in list.
            // entryId: string
            //      The id of entry need to select in list.
            // treeNodeId: string
            //      The id of node need to select in tree.
            // treeNodeType: int
            //      The type (CommerceType enum) of node to highlight
            // tags:
            //		private

            this.catalogEntryList.set("activeSearchResultEntryId", entryId); // tell the other that this highlight action is come from SearchResult onclick
            // highlight node in tree, then the entryList will update (bindData) automatically (base on event)
            this.catalogNodeList.highlightNode(treeNodeId, treeNodeType);
        },

        _onMarketChange: function (value) {
            this.catalogNodeList.set("selectedMarketId", this.marketSelector.item.marketId);
        }
    });
});
