﻿define("commerce/widget/CatalogEntryList", [
// Dojo
"dojo/_base/declare",
"dojo/dom-construct",
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/Deferred",
"dojo/_base/connect",
"dojo/dnd/Source",
"dojo/dom-style",
"dojo/store/Memory",
"dojo/dom-class",
"dojo/dom-geometry",
"dojo/query",
"dojo/aspect",
"dojo/Stateful",

// Dijit
"dijit/_Widget",
"dijit/layout/_LayoutWidget",
"dijit/_TemplatedMixin",
"dijit/_WidgetsInTemplateMixin",

// DGrid
"dgrid/OnDemandGrid",
"dgrid/Selection",
"dgrid/Keyboard",

// EPi Framework
"epi",
"epi/dependency",
"epi/shell/command/_WidgetCommandProviderMixin",
"epi/shell/command/withConfirmation",
"epi/shell/selection",
"epi/shell/dgrid/Formatter",
"epi/shell/_StatefulGetterSetterMixin",

// EPi CMS
"epi-cms/command/CopyContent",
"epi-cms/command/CutContent",
"epi-cms/command/DeleteContent",

"epi-cms/component/command/ChangeContext",
"epi-cms/contentediting/ContentActionSupport",
"commerce/widget/CatalogEntryListViewModel",
"epi/shell/widget/ContextMenu",
"epi-cms/widget/sharedContentDialogHandler",
"epi-cms/dgrid/DnD",

// Resources
"epi/i18n!epi/cms/nls/commerce.components",

// Commerce
"commerce/component/CommerceIdTranslator"
],

function (
// Dojo
    declare,
    domConstruct,
    lang,
    array,
    Deferred,
    connect,
    Source,
    domStyle,
    Memory,
    domClass,
    domGeometry,
    query,
    aspect,
    Stateful,

// Dijit
    _Widget,
    _LayoutWidget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,

// DGrid
    OnDemandGrid,
    dgridSelection,
    Keyboard,

// EPi Framework
    epi,
    dependency,
    _WidgetCommandProviderMixin,
    withConfirmation,
    Selection,
    Formatter,
    _StatefulGetterSetterMixin,

// EPi CMS
    CopyContent,
    CutContent,
    DeleteContent,
    ChangeContext,
    ContentActionSupport,
    CatalogEntryListViewModel,
    ContextMenu,
    sharedContentDialogHandler,
    DnD,

// Resources
    resCommerce,

// Commerce
    CommerceIdTranslator
    ) {

    return declare("commerce.widget.CatalogEntryList", [_Widget, _WidgetCommandProviderMixin, Stateful, _StatefulGetterSetterMixin], {
        // summary: This component displays a list of catalogEntry

        clipboardManager: null,
        typeIdentifiers: ['epi.cms.commerce.catalogentry'],

        _source: null,
        selectedCatalogNode: null,
        activeSearchResultEntryId: null, // current Entry which is selected by SearchResult, need to active/select on grid
        currentContext: null,
        contentActionSupport: null,

        dndTypes: ['epi.commerce.catalogentry'],
        _mouseOverClass: 'epi-dgrid-mouseover',

        // store: [Object]
        //      Represents the REST store to get data.
        store: null,
        marketSelector: null,

        constructor: function () {
            this.selection = new Selection();
        },

        postMixInProperties: function () {
            this.inherited(arguments);

            var getStore = function () {
                var registry = dependency.resolve("epi.storeregistry");
                return registry.get("commerce.objectstore");
            };

            this.store = this.store || getStore();

            this.contentActionSupport = this.contentActionSupport || ContentActionSupport;
        },

        buildRendering: function () {
            // summary:
            //		Construct the UI for this widget with this.domNode initialized as a dgrid.
            // tags:
            //		protected

            this.inherited(arguments);

            // Init catalogentries grid
            var gridClass = declare([OnDemandGrid, dgridSelection, Keyboard, DnD, Formatter]);
            this.grid = new gridClass({
                columns: {
                    icon: {
                        renderCell: function (item, value, node, options) {
                            node.innerHTML = "<span class='dijitInline dijitIcon epi-iconObjectSharedBlock'></span>";
                        }
                    },
                    name: { field: "name", htmlEncode: true, ellipsis: true },
                    "contextmenu": {
                        renderCell: function (item, value, node, options) {
                            var format = "<div class='epi-catalogEntryListEditAction' title='{0}'>\
                                            <span class='dijitInline dijitIcon epi-iconContextMenu' title ='{1}'>&nbsp;</span><div>";
                            node.innerHTML = lang.replace(format, [epi.resources.action.edit, resCommerce.catalogs.catalogentrymenu]);
                        }
                    }
                },
                noDataMessage: resCommerce.catalogs.nocatalogentry,
                selectionMode: "single",
                deselectOnRefresh: false,
                dndSourceType: this.dndTypes,
                dndParams: {
                    creator: lang.hitch(this, this._catalogEntryNodeCreator),
                    copyOnly: true,
                    selfAccept: false,
                    accept: []
                },
                store: this.store
            }, this.domNode);

            // Hide header of the grid
            this.grid.set("showHeader", false);
        },

        postCreate: function () {
            this.inherited(arguments);

            var self = this;

            // whenever catalogNode change, rebind this entryList
            this.watch("selectedCatalogNode", lang.hitch(this, function () {
                this.model.contentData = this.selectedCatalogNode;
                this._bindingData();

            }));

            this.contextMenu = new ContextMenu({ leftClickToOpen: true, category: "context" });
            this.contextMenu.addProvider(this);

            // Handle row click event
            this.grid.on(".dgrid-row:click", function (event) {
                var row = self.grid.row(event);
                self.onSelectionChange(row.data);

            });
            //When mouse is over a row, show the context menu node
            this.grid.on(".dgrid-row:mouseover", function (event) {
                domClass.add(this, self._mouseOverClass);
            });
            //When mouse is out of an unselected row, hide the context menu node
            this.grid.on(".dgrid-row:mouseout", function (event) {
                domClass.remove(this, self._mouseOverClass);
            });
            //Open context menu when click on the context menu icon in row
            this.grid.on(".epi-iconContextMenu:click", function (evt) {
                self.contextMenu.scheduleOpen(this, null, { x: evt.pageX, y: evt.pageY });
            });

            aspect.after(this.getParent(), "resize", lang.hitch(this, this._onResize));

            // after grid.renderArray (it means the _bindingData() is completed) we highlight the selected Item, if needed
            aspect.after(this.grid, "renderArray", lang.hitch(this, function (df) {
                Deferred.when(df, lang.hitch(this, function () {
                    var activeSearchResultEntryId = this.get('activeSearchResultEntryId');
                    if (activeSearchResultEntryId) {
                        this.highlightItem(activeSearchResultEntryId);
                        this.set('activeSearchResultEntryId', null);
                    }
                }));

                return df;  // always have to return when using aspect.after(), aspect.before(), aspect.around()
            }));
        },

        startup: function () {

            if (this._started) {
                return;
            }

            this.inherited(arguments);

            this.model = new CatalogEntryListViewModel({
                store: this.store
            });
            connect.connect(this.model, "onDelete", this, this._onDeleted);
            this._setupCommands();
        },


        _onResize: function () {
            // summary
            //      resize the catalogentry list when its parent is resized.
            // tag
            //      private
            dojo.style(this.domNode, "height", domGeometry.getMarginBox(this.getParent().domNode).h + "px");
        },


        _bindingData: function () {
            // summary
            //      build the new query, set to the grid. The grid will rebind data automatically
            //  tag
            //      private

            var query = { parentLink: this.selectedCatalogNode.contentLink, marketId: this.marketSelector.item.marketId, typeIdentifiers: this.typeIdentifiers };

            // send the search result entry id (if any) along with the query
            var activeSearchResultEntryId = this.get("activeSearchResultEntryId");
            if (activeSearchResultEntryId) {
                var idTranslator = new CommerceIdTranslator();
                var EnumContentType = dependency.resolve("commerce.EnumContentType");
                var contentLinkTranslated = idTranslator.getContentLink(activeSearchResultEntryId, EnumContentType.CatalogEntry);
                lang.mixin(query, { activeSearchResultEntryContentLink: contentLinkTranslated });
            }
            this.grid.set("query", query);
        },

        _setupCommands: function () {

            var settings = {
                category: "context",
                model: this.model,
                selection: this.selection,
                clipboard: this.clipboardManager
            };


            this._editCommand = new ChangeContext({
                label: resCommerce.command.viewcatalogentry,
                category: "context",
                forceContextChange: true
            });

            this.set("commands", [
                this._editCommand,
                withConfirmation(new DeleteContent(settings), sharedContentDialogHandler, { selection: this.selection })
            ]);
        },


        _catalogEntryNodeCreator: function (item, hint) {
            // summary:
            //      Custom DnD avatar creator method

            node = domConstruct.create("div").appendChild(document.createTextNode(item.name));
            return {
                "node": node,
                "type": lang.clone(this.dndTypes),
                "data": item.properties.code    /* data which is carried by the drag and drop to OnPageEdit, ... */
            };
        },


        restore: function () {
            // summary: Restore component visuals
            this.resize();
        },


        onSelectionChange: function (item) {
            // summary:
            //      Update the command models with the current selection.
            // item:
            //      The selected data item (E.g.: row.data)
            // tag:
            //      public callback

            this._editCommand.set("model", item);

            var selected = item ? [{ type: "epi.cms.commerce.catalogentry", data: item}] : [];
            this.selection.set("data", selected);
        },

        _onPaste: function () {

        },

        _onDeleted: function (content) {
            // summary:
            //    Handles event when selecting delete function
            //tags:
            //    private

            var clipboardData = this.clipboardManager.getDataObject(),
                item = clipboardData ? clipboardData[0] : null;

            if (item && item.data.contentLink === content.contentLink) {
                this.clipboardManager.clear();
            }

            this._bindingData();
        },

        _onItemChanged: function (id, item) {
            // summary:
            //      Listen to catalog entry list store to update catalog entry name, etc ...
            // id: Object
            //      Item id
            // item: Object
            //      The updated item
            // tags:
            //      Private

            // Refresh catalog entry list data

            this._bindingData();
        },

        highlightItem: function (entryId) {
            // summary:
            //    Highlight the catalog entry by Commerce entryId.
            // entryId: String
            //    The entryId of a catalog entry.
            // tags:
            //    public

            var dataOfHighlightItem = null;

            if (entryId) {
                this.grid.clearSelection();

                var idTranslator = new CommerceIdTranslator();
                var EnumContentType = dependency.resolve("commerce.EnumContentType");
                var contentLinkTranslated = idTranslator.getContentLink(entryId, EnumContentType.CatalogEntry);
                if (contentLinkTranslated) {
                    var row = this.grid.row(contentLinkTranslated);
                    if (row) {
                        this.grid.select(row);
                        dataOfHighlightItem = row.data;
                    }
                }

                this.onSelectionChange(dataOfHighlightItem);
            }
        },

        showCommands: function () {
            this._setCommandVisbility(true);
        },

        hideCommands: function () {
            this._setCommandVisbility(false);
        },

        _setCommandVisbility: function (visible) {
            array.forEach(this.commands, function (command) {
                command.set("isAvailable", visible);
            });
        }
    });
});
