﻿define("commerce/widget/CatalogNodeList", [

// Dojo
"dojo",
"dojo/dom-class",
"dojo/_base/array",

// Dijit
"dijit/_Widget",
"dijit/_TemplatedMixin",
"dijit/_WidgetsInTemplateMixin",

// Epi
"epi/dependency",
"epi-cms/command/DeleteContent",
"epi-cms/component/ContentFilter",
"epi/shell/widget/ContextMenu",
"epi/shell/command/_WidgetCommandProviderMixin",
"epi/shell/selection",

// Commerce
"commerce/widget/CatalogNodeTree",
"commerce/widget/CatalogNodeTreeStoreModel",
"commerce/component/CommerceIdTranslator"

],

function (
// Dojo
    dojo,
    domClass,
    array,

// Dijit
    _Widget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,

// EPi
    dependency,
    DeleteContent,
    ContentFilter,
    ContextMenu,
    _WidgetCommandProviderMixin,
    Selection,

// Commerce
    CatalogNodeTree,
    CatalogNodeTreeStoreModel,
    CommerceIdTranslator

    ) {

    return dojo.declare("commerce.widget.CatalogNodeList", [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _WidgetCommandProviderMixin], {

        rootMenu: null,
        catalogNodeMenu: null,
        _application: null,

        _renamingNode: null,

        _store: null,
        _commerceRootId: 0,

        isTreeLoaded: null,
        selectedMarketId: null,

        templateString: '<div> \
                             <div data-dojo-attach-point="treeNode" id="${id}_treeNode"></div>\
                         </div>',

        postMixInProperties: function () {
            this.inherited(arguments);
            this._application = dependency.resolve("commerce.Application");
            this._commerceRootId = this._application.getCommerceRoot();
            this.selection = new Selection();
        },

        postCreate: function () {
            this.inherited(arguments);


            this.watch("selectedMarketId", dojo.hitch(this, function () {
                if (!this._tree) {
                    //binding CatalogNode tree with default Market Id
                    this._bindingTree(this.selectedMarketId);
                    //we move the connect here to make sure this._tree is properly initialized
                    this.connect(this._tree, "onResize", dojo.hitch(this, this._onResize));
                    this.connect(this._tree, "onLoad", "_onTreeLoaded");
                    this.connect(this._tree, "onCatalogNodeSelected", "_onTreeNodeSelected");
                    this.connect(this._tree, "_onDelete", "_executeDelete");

                    var model = this._tree.model,
                    settings = {
                        category: "context",
                        model: model,
                        selection: this.selection,
                        clipboard: this._tree.clipboardManager
                    };

                    this._deleteCommand = new DeleteContent(settings);

                    this.set("commands", [
                    this._deleteCommand
                    ]);
                }
                //To reload a tree, treenodes should be cleared from the node map
                var rootItem = this._tree._itemNodesMap[this._commerceRootId];

                 this._tree._itemNodesMap = {};
                 this._tree._itemNodesMap[this._commerceRootId] = rootItem;

                 this._tree.model.setMarketId(this.selectedMarketId);
                 this._tree.selectNode(this._tree.rootNode);
            }));
        },

        _onResize: function () {
            this.onResize();
        },

        onResize: function () {
            // summary:
            //    Raised after the catalog node tree node collapse/expand.
            //
            // tags:
            //    public callback
        },

        startup: function () {

            if (this._started) {
                return;
            }

            this.inherited(arguments);

        },

        _bindingTree: function (selectedMarketId) {
            // summary:
            //      Binding catalogNodes of default market to catalogNode tree
            // tag:
            //      private

            var contextMenu = new ContextMenu({ leftClickToOpen: true, category: "context" });
            contextMenu.addProvider(this);

            this._tree = new CatalogNodeTree({
                selectedMarketId: selectedMarketId,
                model: new CatalogNodeTreeStoreModel(),
                rootNodes: [this._commerceRootId.toString()],
                contextMenu: contextMenu,
                autoExpand: true,
                forceReload: true //force CatalogNodeTree reloaded every time selected node change, regardless last selected node
            }, this.treeNode);
        },

        onCatalogNodeChanged: function (catalognode) {
            // summary:
            //      Event when user click a catalogNode on block catalogNode tree
            // tag:
            //      event

            var selected = catalognode ? [{ type: "epi.cms.commerce.catalognode", data: catalognode.item}] : [];

            this.selection.set("data", selected);
        },

        _onTreeLoaded: function () {
            // select default catalognode root
            this.isTreeLoaded = true;
            this.highlightNode();
        },

        highlightNode: function (nodeId, noteType) {
            // summary:
            //    Highlight for selected contentLink in tree.
            // nodeId: String
            //    The id of the node
            // nodeType: String, Int
            //    The type of the node (which can be cast to CommerceType)
            // tags:
            //    public

            var deferred = new dojo.Deferred();
            
            var contentLink;
            if (!nodeId) {
                contentLink = this._commerceRootId.toString();
            }
            else {
                // because nodeId return by searchResult is 'real' CommerceObject ID, so we need to translate it to ContentId (because with Commerce, we need to use 2MSBit to mask the CommerceContentType)
                var idTranslator = new CommerceIdTranslator();
                var EnumContentType = dependency.resolve("commerce.EnumContentType");
                if (noteType) { // select "Commerce" contentLink, we need to translate the contentLink because NodeTree contains "translated" contentLink
                    contentLink = idTranslator.getContentLink(nodeId, noteType);
                }
            }

            this._tree.selectNodeByItem(contentLink);
            deferred.resolve();
            return deferred;
        },

        _onTreeNodeSelected: function (catalognode) {
            if (catalognode && catalognode.item) {
                // raise event to notify selected folder changed
                this.onCatalogNodeChanged(catalognode);
              
            }
        },

        _onDeleted: function (item) {
            // summary:
            //      Do the rest things after the item is deleted
            // tag:
            //      private

            this.highlightNode(item.parentLink);
        },

        _executeDelete: function () {
            // summary:
            //    Handle delete action on the catalog node tree
            // tags:
            //    private

            this._deleteCommand.execute();
        },

        showCommands: function (isRoot) {
            array.forEach(this.commands, function (command) {
                command.set("isAvailable", !isRoot || command.showForRoot);
            });
        },

        hideCommands: function () {
            array.forEach(this.commands, function (command) {
                command.set("isAvailable", false);
            });
        }
    });
});
