﻿define("commerce/widget/CatalogNodeTree", [

// Dojo
"dojo/_base/array",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/Deferred",
"dojo/json",

// Dijit

// Epi
"epi/dependency",
"commerce/widget/_CatalogNodeTreeNode",
"commerce/widget/CatalogNodeTreeStoreModel",
"epi-cms/widget/ContentTree"
],

function (
    array,
    declare,
    lang,
    Deferred,
    json,


    dependency,
    _CatalogNodeTreeNode,
    CatalogNodeTreeStoreModel,
    ContentTree
    ) {

    return declare("commerce.widget.CatalogNodeTree", [ContentTree], {

        // summary:
        //    The Commerce CatalogNode tree.
        //
        // description:
        //    The CatalogNode tree.
        //
        // tags:
        //    public

        editorWidth: "200px",

        rootNodes: null,

        allowManipulation: false,

        selectedMarketId: null,

        // forceReload: boolean
        //	  set to true if need to force CatalogNodeTree reload even requesting node is selected.
        forceReload: false,

        postMixInProperties: function () {
            // summary:
            //		Post mixin initializations.
            // tags:
            //		protected

            this.inherited(arguments);

            var commerceApplication = dependency.resolve("commerce.Application");
            this.root = commerceApplication.getCommerceRoot();

            //Get the store and attach it to the model
            this.store = dependency.resolve("epi.storeregistry").get("commerce.objectstore");
            //In the first load, the selectedMarketId is the default market Id
            this.model = new CatalogNodeTreeStoreModel({
                store: this.store,
                root: this.root,
                providerName: commerceApplication.getContentProviderName(), 
                selectedMarketId: this.selectedMarketId,
                notAllowToDelete: [this.root]    // id of content which is not allow
            });
        },

        postCreate: function () {
            this.inherited(arguments);

//            this.connect(this.model, "select", this.selectNodeByItem);
//            // Raise onResize event when children change
//            this.connect(this.model, "onChildrenChange", lang.hitch(this, this._onResize));
        },

        isRoot: function (item /*Tree Item*/) {
            if (!this.rootNodes || this.rootNodes.length <= 0) {
                return item === this.rootNode.item;
            }

            var contentLink = item.contentLink.toString();
            var isRoot = false;
            array.some(this.rootNodes, function (rootNode) {
                if (contentLink === rootNode) {
                    isRoot = true;
                    return true;
                }
            });

            return isRoot;
        },

        selectNodeByItem: function (item) {
            // summary:
            //    Find the node by its item, select that node on tree.
            // item:
            //    data item of Tree node
            // tags:
            //    Public

            var nodes = this.getNodesByItem(item);
            if (nodes && nodes.length > 0) {
                var node = nodes[0];
                if (node) {
                    // set selected node to empty, to force the tree to reload even if requesting node is selected node
                    if (this.forceReload) {
                        this.dndController.selectNone();
                    }
                    this.selectNode(node);
                }
            }
        },

        selectNode: function (node) {
            // summary:
            //    Select a given node.
            // node:
            //    Tree node
            // tags:
            //    Public
            if (!node || !node.item) {
                return;
            }

            var path = [node.item.contentLink]; // make an array
            // Expand nodes from child node(catalognode) to root node 
            var parentNode = node;
            do {
                parentNode = parentNode.getParent();
                if (!parentNode || parentNode == this) {
                    break;
                }
                path.push(parentNode.item.contentLink);
            } while (true);
            path = path.reverse(); // path is collection of contentLinks, start with root, then branch/branch/ ..., then node to expand

            var deferredList = this._setPathAttr(path);   // use _setPathAttr() instead of rescusively _expandNode()
            deferredList.then(lang.hitch(this, function () {
                this.focusNode(node);  // scroll into View the selected Node
                this.set("selectedNode", node);

            }));
        },

        catalogEntryContextChanged: function (ctx, callerData) {
            // summary:
            //    Called when the currently loaded catalogEntry changes. I.e. a new catalogEntry is loaded into the preview area.
            // tags:
            //    protected

            // TODO: Refactor to set catalogEntry data when page loaded
        },

        buildNodeFromTemplate: function (args) {
            // summary:
            //		Build tree node from the given template.
            // args: Object
            //      Tree node creation arguments
            // tags:
            //		extension

            var treeNode = new _CatalogNodeTreeNode(args);
            treeNode.bindContextMenu(this.contextMenu);

            this.connect(treeNode, "onTreeNodeSelected", lang.hitch(this, function (catalognode) {
                this.onCatalogNodeSelected(catalognode);
            }));

            this.connect(treeNode, "onResize", lang.hitch(this, this._onResize));

            return treeNode;
        },

        onCatalogNodeSelected: function (catalognode) {
            // summary:
            //      Event when user click a catalog node on catalog node tree
            // tag:
            //      event
        },

        _onKeyPress: function (e) {
            // summary:
            //    Handles onKeyPress event to process cut, copy, paste, delete action.
            // e:
            //    Event object
            // tags:
            //    private

            var widget = e.target && dijit.getEnclosingWidget(e.target);
            if (!widget) { return; }

            // Change event target to prevent losing focus to tree node after press Enter
            e.target = widget.domNode.parentNode;
            this.inherited(arguments);
        },

        _onResize: function () {
            this.onResize();
        },

        onResize: function () {
            // summary:
            //    Raised after the catalog node node collapse/expand.
            //
            // tags:
            //    public callback
        }
    });
});
