﻿define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/_base/Deferred",
    "epi/dependency",
    "epi-cms/widget/ContentTreeStoreModel"
], function (
    declare,
    array,
    lang,
    Deferred,
    dependency,
    ContentTreeStoreModel
) {

    // module:
    //		commerce/component/ProductTreeStoreModel
    // summary:
    //		TODOC

    return declare("commerce.component.CatalogNodeTreeStoreModel", [ContentTreeStoreModel], {
        providerName: null,

        selectedMarketId: null,
        typeIdentifiers: ['epi.cms.commerce.catalog', 'epi.cms.commerce.catalognode'],

        setMarketId: function (marketId) {
            this.selectedMarketId = marketId;
            Deferred.when(this.store.query(this._createQuery({ parentLink: this.root, marketId: marketId }), this._optionIgnore), lang.hitch(this, function (children) {
                //when we refresh catalogs, their children will automatically be refreshed 
                this.onChildrenChange(this.root, children);
            }));
        },

        getChildren: function (parentItem, onComplete) {
            // summary:
            //		Calls onComplete() with array of child items of given parent item, all loaded.

            var id = this.getIdentity(parentItem);
            // Need to append ProviderName for CommerceRoot to direct to CommerceContentProvider to perform
            if (id == this.root) {
                id = id + "__" + this.providerName;
            }
            //TODO: pass the default market Id at first time

            var results = this.store.query(this._createQuery({ parentLink: id, marketId: this.selectedMarketId }), this._optionIgnore);
            var listener = lang.hitch(this, function (item, removedFrom, insertedInto) {
                if (removedFrom === insertedInto) {
                    this.onChange(item);
                }
                Deferred.when(this.store.query(this._createQuery({ parentLink: id, marketId: this.selectedMarketId }), this._optionIgnore), lang.hitch(this, function (children) {
                    this.onChildrenChange(parentItem, children);
                }));
            });

            var observer = this._observers[id];
            if (observer) {
                observer.cancel();
                delete this._observers[id];
            }

            // Observe the query result to update the parent node when have changes.
            this._observers[id] = results.observe(listener, true);

            Deferred.when(results, onComplete);
        },

        canDelete: function (contentItem) {
            return false;

            // TODO: when implement this function, check contentItem for access right to delete this Commerce Object
            // return true if current user can delete it, otherwise, return false.
        }

    });
});
