﻿define("commerce/widget/_CatalogNodeTreeNode", [

// Dojo
"dojo/dom-attr",
"dojo/dom-class",
"dojo/text!./templates/_CatalogNodeTreeNode.html",

"dijit/InlineEditBox",
"dijit/form/TextBox",
"dijit/form/ValidationTextBox",

// Epi
"epi/dependency",
"epi-cms/widget/_ContentTreeNode",

// Resources
    "epi/i18n!epi/cms/nls/commerce.components.catalogs"
],

function (domAttr, domClass, template, InlineEditBox, TextBox, ValidationTextBox, dependency, _ContentTreeNode, resCommerce) {

    dojo.declare("_TreeInlineEditor", [dijit._InlineEditor], {
        postCreate: function () {
            this.inherited(arguments);

            dojo.forEach(["onmousedown", "onmouseup", "onclick", "onkeypress"], function (name) {
                this.connect(this.domNode, name, function (evt) {
                    evt.stopPropagation();
                });
            }, this);
        }
    });

    var LinkTypes = {
        normal: 0,
        root: 1
    };

    return dojo.declare("commerce.widget._CatalogNodeTreeNode", [_ContentTreeNode], {
        // summary:
        //    A customized treenode for block catalogNode tree

        templateString: template,

        _blockCatalogNodeRoot: 'epi-iconObjectFolder',

        _blockCatalogNodeMenu: 'epi-iconContextMenu',

        _blockCatalogNodeDnDIcon: 'epi-iconDnD',

        _blockCatalogNodeExpand: 'epi-iconObjectFolderOpen',

        _blockCatalogNodeCollapse: 'epi-iconObjectFolder',

        isSelected: false,

        isRootNode: undefined,

        _isResetValue: false,

        _menuTooltip: undefined,

        contextMenu: null,

        // _rowStates: Enum flag
        _rowStates: {
            NORMAL: 1,
            EXPANDED: 2,
            ENTERED: 4,
            SELECTED: 8
        },

        _currentRowState: 1, /* NORMAL row state by default */

        postMixInProperties: function () {
            this.inherited(arguments);
            this.isRootNode = this.tree.isRoot(this.item);

            //There will be no menu tooltip for commerce root
            this._menuTooltip = this.isRootNode ? "" : resCommerce.catalognodemenu;
        },

        postCreate: function () {
            this.inherited(arguments);
            dojo.style(this.iconNodeMenu, "zIndex", dojo.position(this.rowNode).zIndex + 1);
            this._updateStyle();
            // Manual set content title to avoid encode trouble
            domAttr.set(this.contentNode, "title", this.label);
        },

        buildRendering: function () {
            this.inherited(arguments);

            var _this = this;
            this.labelWidget = new InlineEditBox({
                value: this.label,
                editorWrapper: "_TreeInlineEditor",
                editor: "dijit.form.ValidationTextBox",
                width: this.tree.editorWidth,
                editorParams: { maxLength: 255 },
                _onMouseOver: function () { /* disable style hover*/ },
                _onClick: function () { /* disable default onclick on editor*/ },
                onChange: function (val) {
                }
            }, this.labelNode);
        },

        _onClick: function (evt) {
            this.inherited(arguments);
            dojo.removeClass(this.labelNode, "dijitInlineEditBoxDisplayModeHover");
        },

        rollbackLabel: function (val) {
            this._isResetValue = true;
            this.labelWidget.set("value", val);
        },

        bindContextMenu: function (menu) {
            if (!this.isRootNode) {
                this.contextMenu = menu;
                this.connect(this.iconNodeMenu, "onclick", "_onIconNodeMenuClick");
            }
        },

        collapse: function () {
            var self = this,
                def = new dojo.Deferred();

            dojo.when(self.inherited(arguments), function () {
                self._currentRowState = self._currentRowState ^ self._rowStates.EXPANDED;
                self._updateStyle();
                // collapse successful trigger event here
                var wipeOut = self._wipeOut;
                wipeOut.onEnd = function () {
                    self.onResize();
                    def.resolve(true);
                };
            });

            return def;
        },

        expand: function () {
            var self = this,
                def = new dojo.Deferred();

            dojo.when(self.inherited(arguments), function () {
                self._currentRowState = self._currentRowState | self._rowStates.EXPANDED;
                self._updateStyle();
                // expand successful trigger event here
                self.onResize();
                def.resolve();
            });

            return def;
        },

        onResize: function () {
            // summary:
            //    Raised after the catalog node tree node collapse/expand.
            //
            // tags:
            //    public callback
        },

        onTreeNodeSelected: function (catalognode) {
            // summary:
            //      public callback when the node is selected
        },

        setSelected: function (/*Boolean*/selected) {
            this.inherited(arguments);
            this.isSelected = selected;

            if (!this.isRootNode) {
                var oldState = this._currentRowState;
                this._currentRowState = selected ?
                (this._currentRowState | this._rowStates.SELECTED) :
                parseInt(this._currentRowState & this._rowStates.SELECTED, 10) > 0 ? (this._currentRowState ^ this._rowStates.SELECTED) : oldState;
                this._updateStyle();
            }
            if (selected) {
                this.onTreeNodeSelected(this);
            }
        },

        _updateBlockCatalogNodeTypeIcon: function () {
            // summary:
            //		Set page type icon for the tree node
            // tags:
            //		private

            if (this.isRootNode) {
                this._iconNodeClass = this._blockCatalogNodeRoot;
            } else {
                this._iconNodeClass = this.isExpanded ? this._blockCatalogNodeExpand : this._blockCatalogNodeCollapse;
            }

            // Add new class
            dojo.addClass(this.iconNode, this._iconNodeClass);
        },

        _onMouseEnter: function (evt) {
            this.inherited(arguments);
            if (!this.isRootNode) {
                this._currentRowState = this._currentRowState | this._rowStates.ENTERED;
                this._updateStyle();
                dojo.removeClass(this.labelNode, "dijitInlineEditBoxDisplayModeHover");
            }
        },

        _onMouseLeave: function (evt) {
            this.inherited(arguments);
            if (!this.isRootNode) {
                this._currentRowState = this._currentRowState ^ this._rowStates.ENTERED;
                this._updateStyle();
            }
        },

        _onIconNodeMenuClick: function (evt) {
            this.contextMenu && this.contextMenu.scheduleOpen(evt.target, null, { x: evt.pageX, y: evt.pageY });
        },

        _updateStyleBySingleState: function (singleRowState) {
            // summary:
            //      Update style for single state
            // singleRowState: Flag
            //      Single (basic) state of the row node
            // tags:
            //      Private

            var states = this._rowStates;
            switch (singleRowState) {
                case states.SELECTED:
                    dojo.addClass(this.iconNodeMenu, this._blockCatalogNodeMenu);
                    break;
                case states.ENTERED:
                    dojo.addClass(this.iconNodeMenu, this._blockCatalogNodeMenu);
                    break;
                default:
                    /* Clear all style */
                    dojo.removeClass(this.iconNodeMenu, this._blockCatalogNodeMenu);
                    dojo.removeClass(this.iconNode, this._blockCatalogNodeExpand);
                    dojo.removeClass(this.iconNode, this._blockCatalogNodeCollapse);

                    /* init icon node */
                    if (this.isRootNode !== undefined) {
                        var nodeClass = this.isExpanded && this.hasChildren() ? this._blockCatalogNodeExpand : this._blockCatalogNodeCollapse;
                        dojo.addClass(this.iconNode, this.isRootNode ? this._blockCatalogNodeRoot : nodeClass);
                    }
                    break;
            }
        },

        _updateStyle: function () {
            // summary:
            //      Update the style by the current state
            // tags:
            //      Private
            for (var singleRowState in this._rowStates) {
                if (parseInt(this._currentRowState & this._rowStates[singleRowState], 10) > 0) {
                    this._updateStyleBySingleState(this._rowStates[singleRowState]);
                }
            }
        }
    });
});
